import 'package:fis_common/event/event_type.dart';
import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:vid/us/vid_us_image.dart';

/// 跨帧测量上下文
abstract class ICrossFrameContext {
  /// 初始图像链接
  String get originUrl;

  /// 当前图像链接
  String get currentUrl;

  /// 初始帧索引
  int get originIndex;

  /// 当前帧索引
  int? get currentIndex;

  /// 当前帧
  VidUsImage? get currentFrame;

  /// 当前图像中 有测量记录的 帧索引 集合
  List<int> get recordIndexList;

  /// 测量项
  IMeasureItem get measureItem;

  /// 是否已结束
  bool get isOver;

  /// 切换图像
  ///
  /// [url] 图像链接
  bool switchImage(String url);

  /// 切换帧
  ///
  /// [frame] 帧数据
  void switchFrame(VidUsImage frame);

  /// 记录当前帧
  ///
  /// [index] 帧索引
  bool recordFrame(int index);

  /// 结束跨帧
  void finish();

  /// 跨帧测量自动结束事件
  late final FEventHandler<void> measureAutoOver;
}
