import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/interfaces/process/items/types.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_shell_measure.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_shell_measure.dart';
import 'dart:math' as math;
// import 'package:fis_measure/process/primitives/urm_straightline.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';

class URMShellDenMeasureCal extends URMShellMeasureCal {
  URMShellDenMeasureCal(URMShellMeasure ref) : super(ref);
  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    super.handleURMSimpleMeasureParams();
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;
      double rimWidth = (ref.meta.rimWidth ?? 0) / 10;

      URMMeasureProcessResult? outResult =
          await urmApplication.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMDenMeasure,
        rOIType: URMROIType.URMShell,
        srcDPoints: param!.srcDPoints,
        shellWidth: rimWidth,
      );
      var result = outResult?.denMeasureResult;

      if (outResult != null && result != null) {
        final feature = ref.feature!;
        if (feature is! ShellImageFeature) return;
        feature.isNeedPaintColors = true;
        if (outResult.outerExterPoints != null) {
          List<DPoint> outerExterPoints = [];
          List<DPoint> interExterPoints = [];
          outResult.outerExterPoints?.forEach((element) {
            outerExterPoints.add(urmApplication.viewToLocal(element));
          });
          param!.srcDPoints?.forEach((element) {
            interExterPoints.add(urmApplication.viewToLocal(element));
          });
          feature.outerExterPoints = outerExterPoints;
          feature.interExterPoints = interExterPoints;
        }
        feature.updateStringValue(
            ItemOutputMeta("Placeholder", "Placeholder", VidUsUnit.None),
            "",
            VidUsUnit.None);
        for (var output in ref.meta.outputs) {
          if (output.name == MeasureTerms.URMDenROI) {
            output.unit = VidUsUnit.percent;
            feature.updateFloatValue(output, result.roiDen * 100, output.unit);
          } else if (output.name == MeasureTerms.URMDenFractalDim) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, result.roiFractalDim, output.unit);
          } else if (output.name == MeasureTerms.URMDenMax) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, result.maxDensity, output.unit);
          } else if (output.name == MeasureTerms.URMDenMin) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, result.minDensity, output.unit);
          } else if (output.name == MeasureTerms.URMDenMean) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, result.meanDensity, output.unit);
          } else if (output.name == MeasureTerms.URMDenStd) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(
                output, math.sqrt(result.varianceDensity), output.unit);
          } else if (output.name == MeasureTerms.Area) {
            output.unit = VidUsUnit.cm2;
            feature.updateFloatValue(output, result.roiArea, output.unit);
          } else if (output.name == MeasureTerms.InURMDenROI) {
            output.unit = VidUsUnit.percent;
            feature.updateFloatValue(
                output, result.inRoiDen * 100, output.unit);
          } else if (output.name == MeasureTerms.InURMDenFractalDim) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(
                output, result.inRoiFractalDim, output.unit);
          } else if (output.name == MeasureTerms.InURMDenMax) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, result.inMaxDensity, output.unit);
          } else if (output.name == MeasureTerms.InURMDenMin) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, result.inMinDensity, output.unit);
          } else if (output.name == MeasureTerms.InURMDenMean) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, result.inMeanDensity, output.unit);
          } else if (output.name == MeasureTerms.InURMDenStd) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(
                output, math.sqrt(result.inVarianceDensity), output.unit);
          } else if (output.name == MeasureTerms.InArea) {
            output.unit = VidUsUnit.cm2;
            feature.updateFloatValue(output, result.inRoiArea, output.unit);
          } else if (output.name == MeasureTerms.OutURMDenROI) {
            output.unit = VidUsUnit.percent;
            feature.updateFloatValue(
                output, result.outRoiDen * 100, output.unit);
          } else if (output.name == MeasureTerms.OutURMDenFractalDim) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(
                output, result.outRoiFractalDim, output.unit);
          } else if (output.name == MeasureTerms.OutURMDenMax) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, result.outMaxDensity, output.unit);
          } else if (output.name == MeasureTerms.OutURMDenMin) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(output, result.outMinDensity, output.unit);
          } else if (output.name == MeasureTerms.OutURMDenMean) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(
                output, result.outMeanDensity, output.unit);
          } else if (output.name == MeasureTerms.OutURMDenStd) {
            output.unit = VidUsUnit.None;
            feature.updateFloatValue(
                output, math.sqrt(result.outVarianceDensity), output.unit);
          } else if (output.name == MeasureTerms.OutArea) {
            output.unit = VidUsUnit.cm2;
            feature.updateFloatValue(output, result.outRoiArea, output.unit);
          }
        }
        urmApplication.onUpdateRimResult?.call(URMRimMesureResult(
          result: outResult,
          measureType: MeasureTypes.URMShellDenMeasure,
        ));
      } else {
        throw Exception("URM Measure API error");
      }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }
}
