import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/interfaces/process/items/types.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_shell_measure.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_shell_measure.dart';

// import 'package:fis_measure/process/primitives/urm_straightline.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';

class URMShellVelMeasureCal extends URMShellMeasureCal {
  URMShellVelMeasureCal(URMShellMeasure ref) : super(ref);
  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    super.handleURMSimpleMeasureParams();
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;
      double rimWidth = (ref.meta.rimWidth ?? 0) / 10;

      URMMeasureProcessResult? outresult =
          await urmApplication.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMVelMeasure,
        rOIType: URMROIType.URMShell,
        srcDPoints: param!.srcDPoints,
        shellWidth: rimWidth,
      );
      if (outresult == null) {
        return;
      }
      URMVelMeasureResult? result = outresult.velMeasureResult;

      if (result != null) {
        final feature = ref.feature!;
        feature.updateStringValue(
            ItemOutputMeta("Placeholder", "Placeholder", VidUsUnit.None),
            "",
            VidUsUnit.None);
        if (feature is! ShellImageFeature) return;
        feature.isNeedPaintColors = true;
        if (outresult.outerExterPoints != null) {
          List<DPoint> outerExterPoints = [];
          List<DPoint> interExterPoints = [];
          outresult.outerExterPoints?.forEach((element) {
            outerExterPoints.add(urmApplication.viewToLocal(element));
          });
          param!.srcDPoints?.forEach((element) {
            interExterPoints.add(urmApplication.viewToLocal(element));
          });
          feature.outerExterPoints = outerExterPoints;
          feature.interExterPoints = interExterPoints;
        }
        for (var output in ref.meta.outputs) {
          if (output.name == MeasureTerms.URMVelMax) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.maxVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelMin) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.minVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelMean) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.meanVel, output.unit);
          } else if (output.name == MeasureTerms.URMVelStd) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.varianceVel, output.unit);
          } else if (output.name == MeasureTerms.Area) {
            output.unit = VidUsUnit.cm2;
            feature.updateFloatValue(output, result.roiArea, output.unit);
          } else if (output.name == MeasureTerms.OutURMVelMax) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.outMaxVel, output.unit);
          } else if (output.name == MeasureTerms.OutURMVelMin) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.outMinVel, output.unit);
          } else if (output.name == MeasureTerms.OutURMVelMean) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.outMeanVel, output.unit);
          } else if (output.name == MeasureTerms.OutURMVelStd) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(
                output, result.outVarianceVel, output.unit);
          } else if (output.name == MeasureTerms.OutArea) {
            output.unit = VidUsUnit.cm2;
            feature.updateFloatValue(output, result.outRoiArea, output.unit);
          } else if (output.name == MeasureTerms.InURMVelMax) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.inMaxVel, output.unit);
          } else if (output.name == MeasureTerms.InURMVelMin) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.inMinVel, output.unit);
          } else if (output.name == MeasureTerms.InURMVelMean) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.inMeanVel, output.unit);
          } else if (output.name == MeasureTerms.InURMVelStd) {
            output.unit = VidUsUnit.mms;
            feature.updateFloatValue(output, result.inVarianceVel, output.unit);
          } else if (output.name == MeasureTerms.InArea) {
            output.unit = VidUsUnit.cm2;
            feature.updateFloatValue(output, result.inRoiArea, output.unit);
          }
          urmApplication.onUpdateRimResult?.call(URMRimMesureResult(
            result: outresult,
            measureType: MeasureTypes.URMShellVelMeasure,
          ));
        }
      } else {
        throw Exception("URM Measure API error");
      }
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }
}
