import 'package:fis_common/logger/logger.dart';
import 'package:fis_jsonrpc/rpc.dart';
import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/items/terms.dart';
import 'package:fis_measure/process/primitives/straightline.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_curvature_measure.dart';
import 'package:fis_measure/process/primitives/urm_measure/urm_straight_curvature_line_measure.dart';
import 'package:fis_measure/process/workspace/urm/application.dart';
import 'package:vid/us/vid_us_unit.dart';
import '../calculator.dart';

/// 自动弯曲度测量
class URMStraightCurvatureLineCal extends Calculator<StraightLine, double> {
  URMStraightCurvatureLineCal(StraightLine ref) : super(ref);
  @override
  void calculate() {}

  @override
  Future<void> calculateAsync() async {
    try {
      if (ref.feature == null) return;
      if (ref.application is! URMApplication) return;
      final URMApplication urmApplication = ref.application as URMApplication;
      final p1 = ref.feature!.startPoint;
      final p2 = ref.feature!.endPoint;

      await calculteURMCurvature(urmApplication, p1, p2);
    } catch (e) {
      logger.e('URM Measure error: $e');
      return;
    }
  }

  // // ✅ URM 测量项 1
  Future<void> calculteURMCurvature(
      URMApplication app, DPoint startPoint, DPoint endPoint) async {
    String description = "URM\n Measuring";
    try {
      URMMeasureProcessResult? result = await app.getURMMeasureResult(
        urmMeasureType: URMMeasureType.URMCurvature,
        rOIType: URMROIType.placeHolder_0,
        srcDPoints: [
          app.localToView(startPoint),
          app.localToView(endPoint),
        ],
      );
      if (result != null) {
        print(
            "URM Measure curvature: ${result.resultData} nums: ${result.resultDPoints?.length}");
        final feature = ref.feature!;
        if (feature is! StraightCurvatureFeature) return;
        feature.autoLinePoints = app.urmPointsToDPoints(result.resultDPoints);
        ItemMeta? meta = feature.refItem.meta;
        var revert = meta.name == URMCurvatureMeasure.revertRatioName;
        if (revert) {
          for (var output in ref.meta.outputs) {
            if (output.name == MeasureTerms.URMCurvature) {
              output.unit = VidUsUnit.None;
              feature.updateFloatValue(
                  output, 1 / result.resultData, output.unit);
            }
          }
        } else {
          for (var output in ref.meta.outputs) {
            if (output.name == MeasureTerms.URMCurvature) {
              output.unit = VidUsUnit.None;
              feature.updateFloatValue(output, result.resultData, output.unit);
            }
          }
        }
        // for (var output in ref.meta.outputs) {
        //   if (output.name == MeasureTerms.URMCurvature) {
        //     output.unit = VidUsUnit.None;
        //     feature.updateFloatValue(output, result.resultData, output.unit);
        //   }
        // }
      } else {
        throw Exception("URM Measure API error");
      }
    } catch (e) {
      description = " ";
      updateStringValue(description);
      return;
    }
    ref.application.updateRenderReady.emit(this, null);
  }
}
