import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/enums/items.dart';
import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/workspace/point_info.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_curvature_trace_measure.dart';
import 'package:fis_measure/process/primitives/area_abstract.dart';
import 'package:fis_measure/process/primitives/trace.dart';
import 'package:fis_measure/utils/prompt_box.dart';
import 'package:fis_measure/utils/canvas.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';

class URMCurvatureTraceMeasure extends Trace {
  URMCurvatureTraceMeasure(ItemMeta meta, IMeasureItem? parent)
      : super(meta, parent);

  static URMCurvatureTraceMeasure createURMCurvatureTraceMeasure(ItemMeta meta,
      [IMeasureItem? parent]) {
    URMCurvatureTraceMeasure trace = URMCurvatureTraceMeasure(meta, parent);
    trace.calculator = URMCurvatureTraceMeasureCal(trace);
    trace.isClosed = false;
    return trace;
  }

  @override
  bool onExecuteMouse(PointInfo args) {
    isAutoSnap = false;
    if (waitingResult) return false;
    if (state == ItemStates.finished) {
      if (args.pointType == PointInfoType.mouseDown) {
        state = ItemStates.waiting;
      }
    }

    if (state == ItemStates.waiting) {
      if (args.pointType == PointInfoType.mouseDown) {
        handleMouseDownWhileWaiting(args);
      }
    } else if (state == ItemStates.running) {
      if (args.pointType == PointInfoType.mouseUp) return false;

      feature?.adopt(args);
      if (args.pointType == PointInfoType.mouseDown) {
        handleFinish();
      } else {
        checkAutoSnap(args);
      }
    }
    return true;
  }

  @override
  void handleMouseDownWhileWaiting(PointInfo args) {
    feature = TraceCurvatureFeature(this);
    if (args.hostVisualArea != null) {
      feature!.hostVisualArea = args.hostVisualArea;
    }
    final point = args.toAreaLogicPoint();
    feature!.adopt(point);
    state = ItemStates.running;
  }

  bool waitingResult = false;

  void handleFinish() async {
    feature!.isActive = false;
    PromptBox.loading("计算中...");
    waitingResult = true;
    await doCalculateAsync();
    doFeatureFinish();
    doFeatureUpdate(); // 若不执行，子测量将无法自动切换
    PromptBox.dismiss();
    waitingResult = false;
  }

  /// 自动结束检测
  @override
  bool checkAutoSnap(PointInfo current, [bool autoFinishFeature = true]) {
    if (feature == null || feature!.innerPoints.length < 3) {
      isSmartMove = false;
      return _syncState(false);
    }

    // final viewport = feature!.hostVisualArea!.viewport!;

    if (isAutoSnap == false) return false;
    final pixelSize = application.displaySize;

    final p1 = feature!.innerPoints.first.scale2Size(pixelSize);
    final p2 = current.scale2Size(pixelSize);
    final length = (p1 - p2).length;
    if (length > snapThreshold * 2.0 && !isSmartMove) {
      isSmartMove = true;
    }
    if (length < snapThreshold && isSmartMove) {
      feature!.innerPoints.last = feature!.innerPoints.first.clone();

      /// 此处的最后一个点，方便计算，但是绘制时要剔除
      // feature!.innerPoints.removeLast();

      HapticFeedback.heavyImpact();
      if (autoFinishFeature) {
        handleFinish();
      }
      isSmartMove = false;
      return _syncState(true);
    } else {
      return _syncState(false);
    }
  }

  bool _syncState(bool isSnap) {
    snapState = isSnap;
    return snapState;
  }
}

class TraceCurvatureFeature extends TraceFeature {
  TraceCurvatureFeature(AreaItemAbstract refItem) : super(refItem);

  List<DPoint> autoLinePoints = [];

  @override
  void paint(Canvas canvas, Size size) {
    super.paint(canvas, size);

    /// 绘制 AutoLine 点集连线
    /// 使用 convert2ViewPoint 转为像素坐标
    final pixelPoints = autoLinePoints.map((e) {
      return convert2ViewPoint(size, e).toOffset();
    }).toList();
    final greenLinePan = Paint()
      ..color = Colors.green
      ..isAntiAlias = true
      ..strokeWidth = 2
      ..style = PaintingStyle.stroke;
    canvas.drawPointsLine(pixelPoints, greenLinePan);
  }
}
