import 'package:fis_measure/interfaces/date_types/point.dart';
import 'package:fis_measure/interfaces/enums/items.dart';
import 'package:fis_measure/interfaces/process/items/item.dart';
import 'package:fis_measure/interfaces/process/items/item_metas.dart';
import 'package:fis_measure/interfaces/process/workspace/point_info.dart';
import 'package:fis_measure/process/calcuators/urm_calcuators/urm_straight_curvature_line.dart';
import 'package:fis_measure/process/primitives/straightline.dart';
import 'package:fis_measure/utils/canvas.dart';
import 'package:fis_measure/utils/prompt_box.dart';
import 'package:flutter/material.dart';

class URMStraightCurvatureLineMeasure extends StraightLine {
  URMStraightCurvatureLineMeasure(ItemMeta meta, IMeasureItem? parent)
      : super(meta, parent);

  static URMStraightCurvatureLineMeasure createURMStraightCurvatureLineMeasure(
      ItemMeta meta,
      [IMeasureItem? parent]) {
    URMStraightCurvatureLineMeasure sraightLine =
        URMStraightCurvatureLineMeasure(meta, parent);
    sraightLine.repeatableEditable = true;
    sraightLine.calculator = URMStraightCurvatureLineCal(sraightLine);
    return sraightLine;
  }

  @override
  bool onExecuteMouse(PointInfo args) {
    if (waitingResult) return false;
    if (state == ItemStates.finished) {
      if (args.pointType == PointInfoType.mouseDown) {
        state = ItemStates.waiting;
      } else {
        return false;
      }
    }

    if (state == ItemStates.waiting) {
      if (args.pointType == PointInfoType.mouseDown) {
        handleMouseDownWhileWaiting(args);
      }
    } else if (state == ItemStates.running) {
      if (args.pointType == PointInfoType.mouseUp) return false;

      feature?.endPoint = args;
      if (args.pointType == PointInfoType.mouseDown) {
        handleFinish();

        /// 重置十字样式
        // changeCrossIndicatorStyle(CrossIndicatorStyle.nomal);
      }
    }
    return true;
  }

  @override
  void handleMouseDownWhileWaiting(PointInfo args) {
    final point = args.toAreaLogicPoint();
    feature = StraightCurvatureFeature(this, point, point);
    if (args.hostVisualArea != null) {
      // handleTissueTM(args.hostVisualArea!.mode.modeType, point);
      feature!.hostVisualArea = args.hostVisualArea;
    }
    state = ItemStates.running;
  }

  bool waitingResult = false;

  void handleFinish() async {
    feature!.isActive = false;
    PromptBox.loading("计算中...");
    waitingResult = true;
    await doCalculateAsync();
    doFeatureFinish();
    doFeatureUpdate(); // 若不执行，子测量将无法自动切换
    PromptBox.dismiss();
    waitingResult = false;
  }
}

class StraightCurvatureFeature extends StraightLineFeature {
  StraightCurvatureFeature(
      IMeasureItem refItem, DPoint startPoint, DPoint endPoint)
      : super(refItem, startPoint, endPoint);

  List<DPoint> autoLinePoints = [];

  @override
  void paint(Canvas canvas, Size size) {
    super.paint(canvas, size);

    /// 绘制 AutoLine 点集连线
    /// 使用 convert2ViewPoint 转为像素坐标
    final pixelPoints = autoLinePoints.map((e) {
      return convert2ViewPoint(size, e).toOffset();
    }).toList();
    final greenLinePan = Paint()
      ..color = Colors.green
      ..isAntiAlias = true
      ..strokeWidth = 2
      ..style = PaintingStyle.stroke;
    canvas.drawPointsLine(pixelPoints, greenLinePan);
  }
}
